@@ -7,7 +7,7 @@ export PYTEST="python3 -m pytest -vv -p no:sugar --color=no"
 (cd "$PWD/autotest/cpp" && make quick_test || echo "error in quick_test")
 
 # install pip and use it to install test dependencies
-sudo sh -c "curl -sSL 'https://bootstrap.pypa.io/get-pip.py' | python3"
+sudo sh -c "curl -sSL 'https://bootstrap.pypa.io/pip/3.6/get-pip.py' | python3"
 sudo pip3 install -U -r "$PWD/autotest/requirements.txt"
 
 # Run all the Python autotests
