@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: EnvisatFile.c 4c86d24f4f400bc03425297f80fa72e62237b715 2019-03-23 23:54:29 +0100 Even Rouault $
+ * $Id: EnvisatFile.c 930aad0b9faed99e277da3cd649d4bef0b13f3c8 2021-08-12 14:52:44 +0200 Even Rouault $
  *
  * Project:  APP ENVISAT Support
  * Purpose:  Low Level Envisat file access (read/write) API.
@@ -7,7 +7,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2001, Atlantis Scientific, Inc.
- * Copyright (c) 2010-2012, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2010-2012, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -35,7 +35,7 @@
 #  include "cpl_conv.h"
 #  include "EnvisatFile.h"
 
-CPL_CVSID("$Id: EnvisatFile.c 4c86d24f4f400bc03425297f80fa72e62237b715 2019-03-23 23:54:29 +0100 Even Rouault $")
+CPL_CVSID("$Id: EnvisatFile.c 930aad0b9faed99e277da3cd649d4bef0b13f3c8 2021-08-12 14:52:44 +0200 Even Rouault $")
 
 #else
 #  include "APP/app.h"
@@ -1301,9 +1301,9 @@ Returns:
 
 int EnvisatFile_GetDatasetInfo( EnvisatFile *self,
                                 int ds_index,
-                                char **ds_name,
-                                char **ds_type,
-                                char **filename,
+                                const char **ds_name,
+                                const char **ds_type,
+                                const char **filename,
                                 int  *ds_offset,
                                 int  *ds_size,
                                 int  *num_dsr,
