@@ -127,6 +127,7 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
                                   double& dfMinY,
                                   double& dfMaxX,
                                   double& dfMaxY,
+                                  double& dfRes,
                                   std::unique_ptr<gdal::TileMatrixSet>& poTM,
                                   int& nZoomLevel,
                                   int& nAlignedLevels)
@@ -137,8 +138,8 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
     if( EQUAL(osTargetSRS, "") && EQUAL(osTilingScheme, "CUSTOM") )
         return false;
 
-    CPLString osExtent(CSLFetchNameValueDef(papszOptions, "EXTENT", ""));
-    CPLString osRes(CSLFetchNameValueDef(papszOptions, "RES", ""));
+    const CPLString osExtent(CSLFetchNameValueDef(papszOptions, "EXTENT", ""));
+    const CPLString osRes(CSLFetchNameValueDef(papszOptions, "RES", ""));
     if( !EQUAL(osTilingScheme, "CUSTOM") )
     {
         poTM = gdal::TileMatrixSet::parse(osTilingScheme);
@@ -162,6 +163,10 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
                     "Unsupported tiling scheme: some levels have variable matrix width");
             return false;
         }
+        if( !osTargetSRS.empty() )
+        {
+            CPLError(CE_Warning, CPLE_AppDefined, "Ignoring TARGET_SRS option");
+        }
         osTargetSRS = poTM->crs();
 
         // "Normalize" SRS as AUTH:CODE
@@ -275,10 +280,18 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
     dfMinY = adfExtent[1];
     dfMaxX = adfExtent[2];
     dfMaxY = adfExtent[3];
-    double dfRes = adfGeoTransform[1];
+    dfRes = adfGeoTransform[1];
 
     if( poTM )
     {
+        if( !osExtent.empty() )
+        {
+            CPLError(CE_Warning, CPLE_AppDefined, "Ignoring EXTENT option");
+        }
+        if( !osRes.empty() )
+        {
+            CPLError(CE_Warning, CPLE_AppDefined, "Ignoring RES option");
+        }
         const bool bInvertAxis =
             oTargetSRS.EPSGTreatsAsLatLong() != FALSE ||
             oTargetSRS.EPSGTreatsAsNorthingEasting() != FALSE;
@@ -418,8 +431,6 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
         dfMinY = dfOriY - nBRTileY * dfTileExtent;
         dfMaxX = dfOriX + nBRTileX * dfTileExtent;
         dfMaxY = dfOriY - nTLTileY * dfTileExtent;
-        nXSize = static_cast<int>(std::round((dfMaxX - dfMinX) / dfRes));
-        nYSize = static_cast<int>(std::round((dfMaxY - dfMinY) / dfRes));
     }
     else if( !osExtent.empty() || !osRes.empty() )
     {
@@ -464,12 +475,13 @@ bool COGGetWarpingCharacteristics(GDALDataset* poSrcDS,
     std::unique_ptr<gdal::TileMatrixSet> poTM;
     int nZoomLevel = 0;
     int nAlignedLevels = 0;
+    double dfRes;
     return COGGetWarpingCharacteristics(poSrcDS,
                                         papszOptions,
                                         osResampling,
                                         osTargetSRS,
                                         nXSize, nYSize,
-                                        dfMinX, dfMinY, dfMaxX, dfMaxY,
+                                        dfMinX, dfMinY, dfMaxX, dfMaxY, dfRes,
                                         poTM, nZoomLevel, nAlignedLevels);
 }
 
@@ -514,6 +526,7 @@ static std::unique_ptr<GDALDataset> CreateReprojectedDS(
                                 const double dfMinY,
                                 const double dfMaxX,
                                 const double dfMaxY,
+                                const double dfRes,
                                 GDALProgressFunc pfnProgress,
                                 void * pProgressData,
                                 double& dfCurPixels,
@@ -540,13 +553,30 @@ static std::unique_ptr<GDALDataset> CreateReprojectedDS(
     papszArg = CSLAddString(papszArg, "-t_srs");
     papszArg = CSLAddString(papszArg, osTargetSRS);
     papszArg = CSLAddString(papszArg, "-te");
-    papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g,", dfMinX));
-    papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g,", dfMinY));
-    papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g,", dfMaxX));
-    papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g,", dfMaxY));
+    papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g", dfMinX));
+    papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g", dfMinY));
+    papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g", dfMaxX));
+    papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g", dfMaxY));
     papszArg = CSLAddString(papszArg, "-ts");
     papszArg = CSLAddString(papszArg, CPLSPrintf("%d", nXSize));
     papszArg = CSLAddString(papszArg, CPLSPrintf("%d", nYSize));
+
+    // to be kept in sync with gdalwarp_lib.cpp
+    constexpr double RELATIVE_ERROR_RES_SHARED_BY_COG_AND_GDALWARP = 1e-8;
+    if (fabs((dfMaxX - dfMinX) / dfRes - nXSize) <= RELATIVE_ERROR_RES_SHARED_BY_COG_AND_GDALWARP &&
+        fabs((dfMaxY - dfMinY) / dfRes - nYSize) <= RELATIVE_ERROR_RES_SHARED_BY_COG_AND_GDALWARP )
+    {
+        // Try to produce exactly square pixels
+        papszArg = CSLAddString(papszArg, "-tr");
+        papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g", dfRes));
+        papszArg = CSLAddString(papszArg, CPLSPrintf("%.18g", dfRes));
+    }
+    else
+    {
+        CPLDebug("COG", "Cannot pass -tr option to GDALWarp() due to extent, "
+                 "size and resolution not consistent enough");
+    }
+
     int bHasNoData = FALSE;
     poSrcDS->GetRasterBand(1)->GetNoDataValue(&bHasNoData);
     if( !bHasNoData && CPLTestBool(CSLFetchNameValueDef(
@@ -692,12 +722,14 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
         double dfTargetMinY = 0;
         double dfTargetMaxX = 0;
         double dfTargetMaxY = 0;
+        double dfRes = 0;
         if( !COGGetWarpingCharacteristics(poCurDS, papszOptions,
                                           osTargetResampling,
                                           osTargetSRS,
                                           nTargetXSize, nTargetYSize,
                                           dfTargetMinX, dfTargetMinY,
                                           dfTargetMaxX, dfTargetMaxY,
+                                          dfRes,
                                           poTM, nZoomLevel, nAlignedLevels) )
         {
             return nullptr;
@@ -754,6 +786,7 @@ GDALDataset* GDALCOGCreator::Create(const char * pszFilename,
                                     nTargetXSize, nTargetYSize,
                                     dfTargetMinX, dfTargetMinY,
                                     dfTargetMaxX, dfTargetMaxY,
+                                    dfRes,
                                     pfnProgress, pProgressData,
                                     dfCurPixels, dfTotalPixelsToProcess);
             if( !m_poReprojectedDS )
