@@ -33,7 +33,7 @@
 
 #include <cmath>
 
-CPL_CVSID("$Id: zmapdataset.cpp fd05fbd0482d8fc5ba286491fbb955f275e2d176 2021-07-14 15:37:18 +0200 Even Rouault $")
+CPL_CVSID("$Id: zmapdataset.cpp 4b46f534fed80d31c3e15c1517169f40694a4a3e 2021-10-14 19:17:37 +0200 Even Rouault $")
 
 /************************************************************************/
 /* ==================================================================== */
@@ -207,7 +207,7 @@ ZMapDataset::ZMapDataset() :
 ZMapDataset::~ZMapDataset()
 
 {
-    FlushCache();
+    FlushCache(true);
     if (fp)
         VSIFCloseL(fp);
 }
