@@ -6,7 +6,7 @@
  *
  ******************************************************************************
  * Copyright (c) 2005, Frank Warmerdam
- * Copyright (c) 2009, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2009, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -49,6 +49,7 @@
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wunknown-pragmas"
 #pragma clang diagnostic ignored "-Wdocumentation"
+#pragma clang diagnostic ignored "-Wold-style-cast"
 #endif
 #include "json.h"
 #ifdef __clang__
@@ -56,7 +57,7 @@
 #endif
 #include "ogrgeojsonwriter.h"
 
-CPL_CVSID("$Id: gdal_rat.cpp 36759 2016-12-09 16:15:51Z goatbar $");
+CPL_CVSID("$Id: gdal_rat.cpp abd9705eecbf1222ff3fe3a5ed19e3b5db6ed93c 2022-04-16 15:50:23 +0200 Even Rouault $")
 
 /**
  * \class GDALRasterAttributeTable
@@ -99,7 +100,7 @@ CPL_CVSID("$Id: gdal_rat.cpp 36759 2016-12-09 16:15:51Z goatbar $");
 /*                      Virtual Destructor                              */
 /************************************************************************/
 
-GDALRasterAttributeTable::~GDALRasterAttributeTable() {}
+GDALRasterAttributeTable::~GDALRasterAttributeTable() = default;
 
 /************************************************************************/
 /*                              ValuesIO()                              */
@@ -165,7 +166,7 @@ CPLErr CPL_STDCALL GDALRATValuesIOAsDouble(
 {
     VALIDATE_POINTER1( hRAT, "GDALRATValuesIOAsDouble", CE_Failure );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         ValuesIO(eRWFlag, iField, iStartRow, iLength, pdfData);
 }
 
@@ -227,7 +228,7 @@ CPLErr CPL_STDCALL GDALRATValuesIOAsInteger(
 {
     VALIDATE_POINTER1( hRAT, "GDALRATValuesIOAsInteger", CE_Failure );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         ValuesIO(eRWFlag, iField, iStartRow, iLength, pnData);
 }
 
@@ -287,13 +288,14 @@ CPLErr GDALRasterAttributeTable::ValuesIO(
  */
 CPLErr CPL_STDCALL GDALRATValuesIOAsString(
     GDALRasterAttributeTableH hRAT, GDALRWFlag eRWFlag,
-    int iField, int iStartRow, int iLength, char **papszStrList )
+    int iField, int iStartRow, int iLength, CSLConstList papszStrList )
 
 {
     VALIDATE_POINTER1( hRAT, "GDALRATValuesIOAsString", CE_Failure );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
-        ValuesIO(eRWFlag, iField, iStartRow, iLength, papszStrList);
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
+        ValuesIO(eRWFlag, iField, iStartRow, iLength,
+        const_cast<char**>(papszStrList));
 }
 
 /************************************************************************/
@@ -333,7 +335,7 @@ GDALRATSetRowCount( GDALRasterAttributeTableH hRAT, int nNewCount )
 {
     VALIDATE_POINTER0( hRAT, "GDALRATSetRowCount" );
 
-    static_cast<GDALRasterAttributeTable *>(hRAT)->SetRowCount( nNewCount );
+    GDALRasterAttributeTable::FromHandle(hRAT)->SetRowCount( nNewCount );
 }
 
 /************************************************************************/
@@ -379,7 +381,7 @@ GDALRATGetRowOfValue( GDALRasterAttributeTableH hRAT, double dfValue )
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetRowOfValue", 0 );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         GetRowOfValue( dfValue );
 }
 
@@ -460,7 +462,7 @@ CPLErr CPL_STDCALL GDALRATCreateColumn( GDALRasterAttributeTableH hRAT,
 {
     VALIDATE_POINTER1( hRAT, "GDALRATCreateColumn", CE_Failure );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         CreateColumn( pszFieldName, eFieldType, eFieldUsage );
 }
 
@@ -506,7 +508,7 @@ GDALRATSetLinearBinning( GDALRasterAttributeTableH hRAT,
 {
     VALIDATE_POINTER1( hRAT, "GDALRATSetLinearBinning", CE_Failure );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         SetLinearBinning( dfRow0Min, dfBinSize );
 }
 
@@ -531,7 +533,7 @@ int GDALRasterAttributeTable::GetLinearBinning(
     CPL_UNUSED double * pdfRow0Min ,
     CPL_UNUSED double * pdfBinSize ) const
 {
-    return FALSE;
+    return false;
 }
 
 /************************************************************************/
@@ -551,10 +553,52 @@ GDALRATGetLinearBinning( GDALRasterAttributeTableH hRAT,
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetLinearBinning", 0 );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         GetLinearBinning( pdfRow0Min, pdfBinSize );
 }
 
+/************************************************************************/
+/*                        GDALRATGetTableType()                         */
+/************************************************************************/
+
+/**
+ * \brief Get Rat Table Type
+ *
+ * @since GDAL 2.4
+ *
+ * This function is the same as the C++ method GDALRasterAttributeTable::GetTableType()
+ */
+GDALRATTableType CPL_STDCALL
+GDALRATGetTableType( GDALRasterAttributeTableH hRAT)
+{
+    VALIDATE_POINTER1( hRAT, "GDALRATGetTableType", GRTT_THEMATIC );
+
+    return GDALDefaultRasterAttributeTable::FromHandle(hRAT)->
+        GetTableType();
+}
+
+/************************************************************************/
+/*                        GDALRATSetTableType()                         */
+/************************************************************************/
+
+/**
+ * \brief Set RAT Table Type
+ *
+ * @since GDAL 2.4
+ *
+ * This function is the same as the C++ method GDALRasterAttributeTable::SetTableType()
+ */
+CPLErr CPL_STDCALL
+GDALRATSetTableType( GDALRasterAttributeTableH hRAT,
+                         const GDALRATTableType eInTableType )
+
+{
+    VALIDATE_POINTER1( hRAT, "GDALRATSetTableType", CE_Failure );
+
+    return GDALDefaultRasterAttributeTable::FromHandle(hRAT)->
+        SetTableType( eInTableType );
+}
+
 /************************************************************************/
 /*                             Serialize()                              */
 /************************************************************************/
@@ -566,10 +610,10 @@ CPLXMLNode *GDALRasterAttributeTable::Serialize() const
 
 {
     if( ( GetColumnCount() == 0 ) && ( GetRowCount() == 0 ) )
-        return NULL;
+        return nullptr;
 
     CPLXMLNode *psTree
-        = CPLCreateXMLNode( NULL, CXT_Element, "GDALRasterAttributeTable" );
+        = CPLCreateXMLNode( nullptr, CXT_Element, "GDALRasterAttributeTable" );
 
 /* -------------------------------------------------------------------- */
 /*      Add attributes with regular binning info if appropriate.        */
@@ -591,6 +635,22 @@ CPLXMLNode *GDALRasterAttributeTable::Serialize() const
             CXT_Text, szValue );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Store table type                                                */
+/* -------------------------------------------------------------------- */
+    const GDALRATTableType tableType = GetTableType();
+    if (tableType == GRTT_ATHEMATIC)
+    {
+        CPLsnprintf( szValue, sizeof(szValue), "athematic" );
+    }
+    else
+    {
+        CPLsnprintf( szValue, sizeof(szValue), "thematic" );
+    }
+    CPLCreateXMLNode(
+        CPLCreateXMLNode( psTree, CXT_Attribute, "tableType" ),
+        CXT_Text, szValue );
+
 /* -------------------------------------------------------------------- */
 /*      Define each column.                                             */
 /* -------------------------------------------------------------------- */
@@ -622,13 +682,13 @@ CPLXMLNode *GDALRasterAttributeTable::Serialize() const
 /*      Write out each row.                                             */
 /* -------------------------------------------------------------------- */
     const int iRowCount = GetRowCount();
-    CPLXMLNode *psTail = NULL;
-    CPLXMLNode *psRow = NULL;
+    CPLXMLNode *psTail = nullptr;
+    CPLXMLNode *psRow = nullptr;
 
     for( int iRow = 0; iRow < iRowCount; iRow++ )
     {
-        psRow = CPLCreateXMLNode( NULL, CXT_Element, "Row" );
-        if( psTail == NULL )
+        psRow = CPLCreateXMLNode( nullptr, CXT_Element, "Row" );
+        if( psTail == nullptr )
             CPLAddXMLChild( psTree, psRow );
         else
             psTail->psNext = psRow;
@@ -679,8 +739,9 @@ void *GDALRasterAttributeTable::SerializeJSON() const
 /* -------------------------------------------------------------------- */
     double dfRow0Min = 0.0;
     double dfBinSize = 0.0;
-    json_object *poRow0Min = NULL;
-    json_object *poBinSize = NULL;
+    json_object *poRow0Min = nullptr;
+    json_object *poBinSize = nullptr;
+    json_object *poTableType = nullptr;
 
     if( GetLinearBinning(&dfRow0Min, &dfBinSize) )
     {
@@ -691,6 +752,20 @@ void *GDALRasterAttributeTable::SerializeJSON() const
         json_object_object_add( poRAT, "binSize", poBinSize );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Table Type                                                      */
+/* -------------------------------------------------------------------- */
+    const GDALRATTableType tableType = GetTableType();
+    if (tableType == GRTT_ATHEMATIC)
+    {
+        poTableType = json_object_new_string( "athematic" );
+    }
+    else
+    {
+        poTableType = json_object_new_string( "thematic" );
+    }
+    json_object_object_add( poRAT, "tableType", poTableType);
+
 /* -------------------------------------------------------------------- */
 /*      Define each column.                                             */
 /* -------------------------------------------------------------------- */
@@ -738,7 +813,7 @@ void *GDALRasterAttributeTable::SerializeJSON() const
 
         for( int iCol = 0; iCol < iColCount; iCol++ )
         {
-            json_object *poF = NULL;
+            json_object *poF = nullptr;
             if( GetTypeOfCol(iCol) == GFT_Integer )
                 poF = json_object_new_int( GetValueAsInt(iRow, iCol) );
             else if( GetTypeOfCol(iCol) == GFT_Real )
@@ -774,19 +849,36 @@ CPLErr GDALRasterAttributeTable::XMLInit( CPLXMLNode *psTree,
 /* -------------------------------------------------------------------- */
 /*      Linear binning.                                                 */
 /* -------------------------------------------------------------------- */
-    if( CPLGetXMLValue( psTree, "Row0Min", NULL )
-        && CPLGetXMLValue( psTree, "BinSize", NULL ) )
+    if( CPLGetXMLValue( psTree, "Row0Min", nullptr )
+        && CPLGetXMLValue( psTree, "BinSize", nullptr ) )
     {
         SetLinearBinning( CPLAtof(CPLGetXMLValue( psTree, "Row0Min","" )),
                           CPLAtof(CPLGetXMLValue( psTree, "BinSize","" )) );
     }
 
+/* -------------------------------------------------------------------- */
+/*      Table Type                                                      */
+/* -------------------------------------------------------------------- */
+    if( CPLGetXMLValue( psTree, "tableType", nullptr ) )
+    {
+        const char* pszValue = CPLGetXMLValue(psTree, "tableType", "thematic");
+        if (EQUAL(pszValue, "athematic"))
+        {
+            SetTableType(GRTT_ATHEMATIC);
+        }
+        else
+        {
+            SetTableType(GRTT_THEMATIC);
+        }
+    }
+
+
 /* -------------------------------------------------------------------- */
 /*      Column definitions                                              */
 /* -------------------------------------------------------------------- */
 
     for( CPLXMLNode *psChild = psTree->psChild;
-         psChild != NULL;
+         psChild != nullptr;
          psChild = psChild->psNext)
     {
         if( psChild->eType == CXT_Element
@@ -805,7 +897,7 @@ CPLErr GDALRasterAttributeTable::XMLInit( CPLXMLNode *psTree,
 /*      Row data.                                                       */
 /* -------------------------------------------------------------------- */
     for( CPLXMLNode *psChild = psTree->psChild;
-         psChild != NULL;
+         psChild != nullptr;
          psChild = psChild->psNext)
     {
         if( psChild->eType == CXT_Element
@@ -815,13 +907,13 @@ CPLErr GDALRasterAttributeTable::XMLInit( CPLXMLNode *psTree,
             int iField = 0;
 
             for( CPLXMLNode *psF = psChild->psChild;
-                 psF != NULL;
+                 psF != nullptr;
                  psF = psF->psNext )
             {
                 if( psF->eType != CXT_Element || !EQUAL(psF->pszValue,"F") )
                     continue;
 
-                if( psF->psChild != NULL && psF->psChild->eType == CXT_Text )
+                if( psF->psChild != nullptr && psF->psChild->eType == CXT_Text )
                     SetValue( iRow, iField++, psF->psChild->pszValue );
                 else
                     SetValue( iRow, iField++, "" );
@@ -911,8 +1003,8 @@ GDALRATInitializeFromColorTable( GDALRasterAttributeTableH hRAT,
 {
     VALIDATE_POINTER1( hRAT, "GDALRATInitializeFromColorTable", CE_Failure );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
-        InitializeFromColorTable( static_cast<GDALColorTable *>(hCT) );
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
+        InitializeFromColorTable( GDALColorTable::FromHandle(hCT) );
 }
 
 /************************************************************************/
@@ -945,7 +1037,7 @@ GDALColorTable *GDALRasterAttributeTable::TranslateToColorTable(
     const int iBlue = GetColOfUsage( GFU_Blue );
 
     if( iRed == -1 || iGreen == -1 || iBlue == -1 )
-        return NULL;
+        return nullptr;
 
     const int iAlpha = GetColOfUsage( GFU_Alpha );
 
@@ -960,14 +1052,16 @@ GDALColorTable *GDALRasterAttributeTable::TranslateToColorTable(
             iMaxCol = GetColOfUsage( GFU_MinMax );
 
         if( iMaxCol == -1 || GetRowCount() == 0 )
-            return NULL;
+            return nullptr;
 
         for( int iRow = 0; iRow < GetRowCount(); iRow++ )
+        {
             nEntryCount =
-                std::max(nEntryCount, GetValueAsInt(iRow, iMaxCol) + 1);
+                std::max(nEntryCount, std::min(65535, GetValueAsInt(iRow, iMaxCol)) + 1);
+        }
 
         if( nEntryCount < 0 )
-            return NULL;
+            return nullptr;
 
         // Restrict our number of entries to something vaguely sensible.
         nEntryCount = std::min(65535, nEntryCount);
@@ -1015,9 +1109,9 @@ GDALRATTranslateToColorTable( GDALRasterAttributeTableH hRAT,
                               int nEntryCount )
 
 {
-    VALIDATE_POINTER1( hRAT, "GDALRATTranslateToColorTable", NULL );
+    VALIDATE_POINTER1( hRAT, "GDALRATTranslateToColorTable", nullptr );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         TranslateToColorTable( nEntryCount );
 }
 
@@ -1044,7 +1138,7 @@ void GDALRasterAttributeTable::DumpReadable( FILE * fp )
 
     CPLDestroyXMLNode( psTree );
 
-    if( fp == NULL )
+    if( fp == nullptr )
         fp = stdout;
 
     fprintf( fp, "%s\n", pszXMLText );
@@ -1068,7 +1162,7 @@ GDALRATDumpReadable( GDALRasterAttributeTableH hRAT, FILE *fp )
 {
     VALIDATE_POINTER0( hRAT, "GDALRATDumpReadable" );
 
-    static_cast<GDALRasterAttributeTable *>(hRAT)->DumpReadable( fp );
+    GDALRasterAttributeTable::FromHandle(hRAT)->DumpReadable( fp );
 }
 
 /* \class GDALDefaultRasterAttributeTable
@@ -1087,10 +1181,11 @@ GDALRATDumpReadable( GDALRasterAttributeTableH hRAT, FILE *fp )
 //! Construct empty table.
 
 GDALDefaultRasterAttributeTable::GDALDefaultRasterAttributeTable() :
-    bLinearBinning(FALSE),
+    bLinearBinning(false),
     dfRow0Min(-0.5),
     dfBinSize(1.0),
-    bColumnsAnalysed(FALSE),
+    eTableType(GRTT_THEMATIC),
+    bColumnsAnalysed(false),
     nMinCol(-1),
     nMaxCol(-1),
     nRowCount(0)
@@ -1112,27 +1207,13 @@ GDALRasterAttributeTableH CPL_STDCALL GDALCreateRasterAttributeTable()
     return new GDALDefaultRasterAttributeTable();
 }
 
-/************************************************************************/
-/*                  GDALDefaultRasterAttributeTable()                   */
-/************************************************************************/
-
-//! Copy constructor.
-
-GDALDefaultRasterAttributeTable::GDALDefaultRasterAttributeTable(
-    const GDALDefaultRasterAttributeTable &oOther ) : GDALRasterAttributeTable()
-
-{
-    // We have tried to be careful to allow wholesale assignment
-    *this = oOther;
-}
-
 /************************************************************************/
 /*                 ~GDALDefaultRasterAttributeTable()                   */
 /*                                                                      */
 /*      All magic done by magic by the container destructors.           */
 /************************************************************************/
 
-GDALDefaultRasterAttributeTable::~GDALDefaultRasterAttributeTable() {}
+GDALDefaultRasterAttributeTable::~GDALDefaultRasterAttributeTable() = default;
 
 /************************************************************************/
 /*                  GDALDestroyRasterAttributeTable()                   */
@@ -1148,8 +1229,8 @@ void CPL_STDCALL
 GDALDestroyRasterAttributeTable( GDALRasterAttributeTableH hRAT )
 
 {
-    if( hRAT != NULL )
-        delete static_cast<GDALRasterAttributeTable *>(hRAT);
+    if( hRAT != nullptr )
+        delete GDALRasterAttributeTable::FromHandle(hRAT);
 }
 
 /************************************************************************/
@@ -1162,7 +1243,7 @@ GDALDestroyRasterAttributeTable( GDALRasterAttributeTableH hRAT )
 void GDALDefaultRasterAttributeTable::AnalyseColumns()
 
 {
-    bColumnsAnalysed = TRUE;
+    bColumnsAnalysed = true;
 
     nMinCol = GetColOfUsage( GFU_Min );
     if( nMinCol == -1 )
@@ -1198,7 +1279,7 @@ int CPL_STDCALL GDALRATGetColumnCount( GDALRasterAttributeTableH hRAT )
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetColumnCount", 0 );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->GetColumnCount();
+    return GDALRasterAttributeTable::FromHandle(hRAT)->GetColumnCount();
 }
 
 /************************************************************************/
@@ -1212,7 +1293,7 @@ int CPL_STDCALL GDALRATGetColumnCount( GDALRasterAttributeTableH hRAT )
 const char *GDALDefaultRasterAttributeTable::GetNameOfCol( int iCol ) const
 
 {
-    if( iCol < 0 || iCol >= (int) aoFields.size() )
+    if( iCol < 0 || iCol >= static_cast<int>(aoFields.size()) )
         return "";
 
     return aoFields[iCol].sName;
@@ -1235,9 +1316,9 @@ const char *CPL_STDCALL GDALRATGetNameOfCol( GDALRasterAttributeTableH hRAT,
                                              int iCol )
 
 {
-    VALIDATE_POINTER1( hRAT, "GDALRATGetNameOfCol", NULL );
+    VALIDATE_POINTER1( hRAT, "GDALRATGetNameOfCol", nullptr );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->GetNameOfCol( iCol );
+    return GDALRasterAttributeTable::FromHandle(hRAT)->GetNameOfCol( iCol );
 }
 
 /************************************************************************/
@@ -1254,7 +1335,7 @@ GDALRATFieldUsage GDALDefaultRasterAttributeTable::GetUsageOfCol(
     int iCol ) const
 
 {
-    if( iCol < 0 || iCol >= (int) aoFields.size() )
+    if( iCol < 0 || iCol >= static_cast<int>(aoFields.size()) )
         return GFU_Generic;
 
     return aoFields[iCol].eUsage;
@@ -1279,7 +1360,7 @@ GDALRATGetUsageOfCol( GDALRasterAttributeTableH hRAT, int iCol )
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetUsageOfCol", GFU_Generic );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->GetUsageOfCol( iCol );
+    return GDALRasterAttributeTable::FromHandle(hRAT)->GetUsageOfCol( iCol );
 }
 
 /************************************************************************/
@@ -1295,7 +1376,7 @@ GDALRATGetUsageOfCol( GDALRasterAttributeTableH hRAT, int iCol )
 GDALRATFieldType GDALDefaultRasterAttributeTable::GetTypeOfCol( int iCol ) const
 
 {
-    if( iCol < 0 || iCol >= (int) aoFields.size() )
+    if( iCol < 0 || iCol >= static_cast<int>(aoFields.size()) )
         return GFT_Integer;
 
     return aoFields[iCol].eType;
@@ -1320,7 +1401,7 @@ GDALRATGetTypeOfCol( GDALRasterAttributeTableH hRAT, int iCol )
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetTypeOfCol", GFT_Integer );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->GetTypeOfCol( iCol );
+    return GDALRasterAttributeTable::FromHandle(hRAT)->GetTypeOfCol( iCol );
 }
 
 /************************************************************************/
@@ -1360,7 +1441,7 @@ GDALRATGetColOfUsage( GDALRasterAttributeTableH hRAT, GDALRATFieldUsage eUsage )
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetColOfUsage", 0 );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         GetColOfUsage( eUsage );
 }
 
@@ -1389,7 +1470,7 @@ GDALRATGetRowCount( GDALRasterAttributeTableH hRAT )
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetRowCount", 0 );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->GetRowCount();
+    return GDALRasterAttributeTable::FromHandle(hRAT)->GetRowCount();
 }
 
 /************************************************************************/
@@ -1400,7 +1481,7 @@ const char *
 GDALDefaultRasterAttributeTable::GetValueAsString( int iRow, int iField ) const
 
 {
-    if( iField < 0 || iField >= (int) aoFields.size() )
+    if( iField < 0 || iField >= static_cast<int>(aoFields.size()) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "iField (%d) out of range.", iField );
@@ -1454,9 +1535,9 @@ const char * CPL_STDCALL
 GDALRATGetValueAsString( GDALRasterAttributeTableH hRAT, int iRow, int iField )
 
 {
-    VALIDATE_POINTER1( hRAT, "GDALRATGetValueAsString", NULL );
+    VALIDATE_POINTER1( hRAT, "GDALRATGetValueAsString", nullptr );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         GetValueAsString(iRow, iField);
 }
 
@@ -1468,7 +1549,7 @@ int
 GDALDefaultRasterAttributeTable::GetValueAsInt( int iRow, int iField ) const
 
 {
-    if( iField < 0 || iField >= (int) aoFields.size() )
+    if( iField < 0 || iField >= static_cast<int>(aoFields.size()) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "iField (%d) out of range.", iField );
@@ -1515,7 +1596,7 @@ GDALRATGetValueAsInt( GDALRasterAttributeTableH hRAT, int iRow, int iField )
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetValueAsInt", 0 );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         GetValueAsInt( iRow, iField );
 }
 
@@ -1527,7 +1608,7 @@ double
 GDALDefaultRasterAttributeTable::GetValueAsDouble( int iRow, int iField ) const
 
 {
-    if( iField < 0 || iField >= (int) aoFields.size() )
+    if( iField < 0 || iField >= static_cast<int>(aoFields.size()) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "iField (%d) out of range.", iField );
@@ -1574,7 +1655,7 @@ GDALRATGetValueAsDouble( GDALRasterAttributeTableH hRAT, int iRow, int iField )
 {
     VALIDATE_POINTER1( hRAT, "GDALRATGetValueAsDouble", 0 );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         GetValueAsDouble(iRow,iField);
 }
 
@@ -1591,20 +1672,20 @@ void GDALDefaultRasterAttributeTable::SetRowCount( int nNewCount )
     if( nNewCount == nRowCount )
         return;
 
-    for( unsigned int iField = 0; iField < aoFields.size(); iField++ )
+    for( auto& oField: aoFields )
     {
-        switch( aoFields[iField].eType )
+        switch( oField.eType )
         {
           case GFT_Integer:
-            aoFields[iField].anValues.resize( nNewCount );
+            oField.anValues.resize( nNewCount );
             break;
 
           case GFT_Real:
-            aoFields[iField].adfValues.resize( nNewCount );
+            oField.adfValues.resize( nNewCount );
             break;
 
           case GFT_String:
-            aoFields[iField].aosValues.resize( nNewCount );
+            oField.aosValues.resize( nNewCount );
             break;
         }
     }
@@ -1625,7 +1706,7 @@ void GDALDefaultRasterAttributeTable::SetValue( int iRow, int iField,
                                                 const char *pszValue )
 
 {
-    if( iField < 0 || iField >= (int) aoFields.size() )
+    if( iField < 0 || iField >= static_cast<int>(aoFields.size()) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "iField (%d) out of range.", iField );
@@ -1681,7 +1762,7 @@ GDALRATSetValueAsString( GDALRasterAttributeTableH hRAT, int iRow, int iField,
 {
     VALIDATE_POINTER0( hRAT, "GDALRATSetValueAsString" );
 
-    static_cast<GDALRasterAttributeTable *>(hRAT)->
+    GDALRasterAttributeTable::FromHandle(hRAT)->
         SetValue( iRow, iField, pszValue );
 }
 
@@ -1750,7 +1831,7 @@ GDALRATSetValueAsInt( GDALRasterAttributeTableH hRAT, int iRow, int iField,
 {
     VALIDATE_POINTER0( hRAT, "GDALRATSetValueAsInt" );
 
-    static_cast<GDALRasterAttributeTable *>(hRAT)->
+    GDALRasterAttributeTable::FromHandle(hRAT)->
         SetValue( iRow, iField, nValue);
 }
 
@@ -1762,7 +1843,7 @@ void GDALDefaultRasterAttributeTable::SetValue( int iRow, int iField,
                                                 double dfValue )
 
 {
-    if( iField < 0 || iField >= (int) aoFields.size() )
+    if( iField < 0 || iField >= static_cast<int>(aoFields.size()) )
     {
         CPLError( CE_Failure, CPLE_AppDefined,
                   "iField (%d) out of range.", iField );
@@ -1784,7 +1865,7 @@ void GDALDefaultRasterAttributeTable::SetValue( int iRow, int iField,
     switch( aoFields[iField].eType )
     {
       case GFT_Integer:
-        aoFields[iField].anValues[iRow] = (int) dfValue;
+        aoFields[iField].anValues[iRow] = static_cast<int>(dfValue);
         break;
 
       case GFT_Real:
@@ -1819,7 +1900,7 @@ GDALRATSetValueAsDouble( GDALRasterAttributeTableH hRAT, int iRow, int iField,
 {
     VALIDATE_POINTER0( hRAT, "GDALRATSetValueAsDouble" );
 
-    static_cast<GDALRasterAttributeTable *>(hRAT)->
+    GDALRasterAttributeTable::FromHandle(hRAT)->
         SetValue( iRow, iField, dfValue );
 }
 
@@ -1831,7 +1912,7 @@ int GDALDefaultRasterAttributeTable::ChangesAreWrittenToFile()
 {
     // GDALRasterBand.SetDefaultRAT needs to be called on instances of
     // GDALDefaultRasterAttributeTable since changes are just in-memory
-    return FALSE;
+    return false;
 }
 
 /************************************************************************/
@@ -1848,9 +1929,9 @@ int GDALDefaultRasterAttributeTable::ChangesAreWrittenToFile()
 int CPL_STDCALL
 GDALRATChangesAreWrittenToFile( GDALRasterAttributeTableH hRAT )
 {
-    VALIDATE_POINTER1( hRAT, "GDALRATChangesAreWrittenToFile", FALSE );
+    VALIDATE_POINTER1( hRAT, "GDALRATChangesAreWrittenToFile", false );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->
+    return GDALRasterAttributeTable::FromHandle(hRAT)->
         ChangesAreWrittenToFile();
 }
 
@@ -1883,24 +1964,24 @@ int GDALDefaultRasterAttributeTable::GetRowOfValue( double dfValue ) const
     if( nMinCol == -1 && nMaxCol == -1 )
         return -1;
 
-    const GDALRasterAttributeField *poMin = NULL;
+    const GDALRasterAttributeField *poMin = nullptr;
     if( nMinCol != -1 )
         poMin = &(aoFields[nMinCol]);
     else
-        poMin = NULL;
+        poMin = nullptr;
 
-    const GDALRasterAttributeField *poMax = NULL;
+    const GDALRasterAttributeField *poMax = nullptr;
     if( nMaxCol != -1 )
         poMax = &(aoFields[nMaxCol]);
     else
-        poMax = NULL;
+        poMax = nullptr;
 
 /* -------------------------------------------------------------------- */
 /*      Search through rows for match.                                  */
 /* -------------------------------------------------------------------- */
     for( int iRow = 0; iRow < nRowCount; iRow++ )
     {
-        if( poMin != NULL )
+        if( poMin != nullptr )
         {
             if( poMin->eType == GFT_Integer )
             {
@@ -1917,7 +1998,7 @@ int GDALDefaultRasterAttributeTable::GetRowOfValue( double dfValue ) const
                 break;
         }
 
-        if( poMax != NULL )
+        if( poMax != nullptr )
         {
             if( (poMax->eType == GFT_Integer
                  && dfValue > poMax->anValues[iRow] ) ||
@@ -1953,7 +2034,7 @@ CPLErr GDALDefaultRasterAttributeTable::SetLinearBinning( double dfRow0MinIn,
                                                           double dfBinSizeIn )
 
 {
-    bLinearBinning = TRUE;
+    bLinearBinning = true;
     dfRow0Min = dfRow0MinIn;
     dfBinSize = dfBinSizeIn;
 
@@ -1969,14 +2050,60 @@ int GDALDefaultRasterAttributeTable::GetLinearBinning(
 
 {
     if( !bLinearBinning )
-        return FALSE;
+        return false;
 
     *pdfRow0Min = dfRow0Min;
     *pdfBinSize = dfBinSize;
 
-    return TRUE;
+    return true;
 }
 
+/************************************************************************/
+/*                          GetTableType()                              */
+/************************************************************************/
+
+/**
+ * \brief Get RAT Table Type
+ *
+ * Returns whether table type is thematic or athematic
+ *
+ * This method is the same as the C function GDALRATGetTableType().
+ *
+ * @since GDAL 2.4
+ *
+ * @return GRTT_THEMATIC or GRTT_ATHEMATIC
+ */
+
+GDALRATTableType GDALDefaultRasterAttributeTable::GetTableType() const
+{
+    return eTableType;
+}
+
+/************************************************************************/
+/*                          SetTableType()                              */
+/************************************************************************/
+
+/**
+ * \brief Set RAT Table Type
+ *
+ * Set whether table type is thematic or athematic
+ *
+ * This method is the same as the C function GDALRATSetTableType().
+ *
+ * @param eInTableType the new RAT table type (GRTT_THEMATIC or GRTT_ATHEMATIC)
+ *
+ * @since GDAL 2.4
+ *
+ * @return CE_None on success or CE_Failure on failure.
+ */
+
+CPLErr GDALDefaultRasterAttributeTable::SetTableType(const GDALRATTableType eInTableType)
+{
+    eTableType = eInTableType;
+    return CE_None;
+}
+
+
 /************************************************************************/
 /*                            CreateColumn()                            */
 /************************************************************************/
@@ -2012,6 +2139,57 @@ CPLErr GDALDefaultRasterAttributeTable::CreateColumn(
     return CE_None;
 }
 
+/************************************************************************/
+/*                            RemoveStatistics()                        */
+/************************************************************************/
+
+/**
+ * \brief Remove Statistics from RAT
+ *
+ * Remove statistics (such as histogram) from the RAT. This is important
+ * if these have been invalidated, for example by cropping the image.
+ *
+ * This method is the same as the C function GDALRATRemoveStatistics().
+ *
+ * @since GDAL 2.4
+ */
+
+void GDALDefaultRasterAttributeTable::RemoveStatistics()
+
+{
+    // since we are storing the fields in a vector it will generally
+    // be faster to create a new vector and replace the old one
+    // rather than actually erasing columns.
+    std::vector<GDALRasterAttributeField> aoNewFields;
+    for ( const auto& field : aoFields )
+    {
+        switch (field.eUsage)
+        {
+            case GFU_PixelCount:
+            case GFU_Min:
+            case GFU_Max:
+            case GFU_RedMin:
+            case GFU_GreenMin:
+            case GFU_BlueMin:
+            case GFU_AlphaMin:
+            case GFU_RedMax:
+            case GFU_GreenMax:
+            case GFU_BlueMax:
+            case GFU_AlphaMax:
+            {
+                break;
+            }
+
+            default:
+                if (field.sName != "Histogram")
+                {
+                    aoNewFields.push_back(field);
+                }
+        }
+    }
+    aoFields = aoNewFields;
+}
+
 /************************************************************************/
 /*                               Clone()                                */
 /************************************************************************/
@@ -2032,12 +2210,12 @@ GDALDefaultRasterAttributeTable *GDALDefaultRasterAttributeTable::Clone() const
  * This function is the same as the C++ method GDALRasterAttributeTable::Clone()
  */
 GDALRasterAttributeTableH CPL_STDCALL
-GDALRATClone( GDALRasterAttributeTableH hRAT )
+GDALRATClone( const GDALRasterAttributeTableH hRAT )
 
 {
-    VALIDATE_POINTER1( hRAT, "GDALRATClone", NULL );
+    VALIDATE_POINTER1( hRAT, "GDALRATClone", nullptr );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->Clone();
+    return GDALRasterAttributeTable::FromHandle(hRAT)->Clone();
 }
 
 /************************************************************************/
@@ -2054,7 +2232,28 @@ void* CPL_STDCALL
 GDALRATSerializeJSON( GDALRasterAttributeTableH hRAT )
 
 {
-    VALIDATE_POINTER1( hRAT, "GDALRATSerializeJSON", NULL );
+    VALIDATE_POINTER1( hRAT, "GDALRATSerializeJSON", nullptr );
 
-    return static_cast<GDALRasterAttributeTable *>(hRAT)->SerializeJSON();
+    return GDALRasterAttributeTable::FromHandle(hRAT)->SerializeJSON();
+}
+
+
+/************************************************************************/
+/*                        GDALRATRemoveStatistics()                     */
+/************************************************************************/
+
+/**
+ * \brief Remove Statistics from RAT
+ *
+ * This function is the same as the C++ method GDALRasterAttributeTable::RemoveStatistics()
+ *
+ * @since GDAL 2.4
+ */
+void CPL_STDCALL
+GDALRATRemoveStatistics( GDALRasterAttributeTableH hRAT )
+
+{
+    VALIDATE_POINTER0( hRAT, "GDALRATRemoveStatistics" );
+
+    GDALRasterAttributeTable::FromHandle(hRAT)->RemoveStatistics();
 }
