@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_core.h baf6754741beb24089c3895bdb6de8f9493000e8 2021-09-18 14:06:41 +0200 Even Rouault $
+ * $Id: ogr_core.h 3b0c26d57528326c074a9c82890e737e2d516a7d 2022-02-23 14:39:49Z Andrew C Aitchison $
  *
  * Project:  OpenGIS Simple Features Reference Implementation
  * Purpose:  Define some core portability services for cross-platform OGR code.
@@ -151,6 +151,29 @@ class CPL_DLL OGREnvelope
         return MinX <= other.MinX && MinY <= other.MinY &&
                MaxX >= other.MaxX && MaxY >= other.MaxY;
     }
+
+    /** Return whether the current rectangle is equal to the other rectangle */
+    bool operator== (const OGREnvelope& other) const
+    {
+#ifdef HAVE_GCC_DIAGNOSTIC_PUSH
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wfloat-equal"
+#endif
+        return MinX == other.MinX &&
+               MinY == other.MinY &&
+               MaxX == other.MaxX &&
+               MaxY == other.MaxY;
+
+#ifdef HAVE_GCC_DIAGNOSTIC_PUSH
+#pragma GCC diagnostic pop
+#endif
+    }
+
+    /** Return whether the current rectangle is not equal to the other rectangle */
+    bool operator!= (const OGREnvelope& other) const
+    {
+        return !(*this == other);
+    }
 };
 
 } // extern "C++"
