@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: ogr_arcgen.h 842d122d2f23aaebb28362e083b52d6bc7dbcde2 2019-08-11 17:42:34 +0200 Even Rouault $
+ * $Id: ogr_arcgen.h bc3d9f5351962c422f3e57a9ab1a251d91659192 2020-05-09 21:07:14 +0200 Even Rouault $
  *
  * Project:  Arc/Info Generate Translator
  * Purpose:  Definition of classes for OGR .arcgen driver.
@@ -33,10 +33,10 @@
 #include "ogrsf_frmts.h"
 
 /************************************************************************/
-/*                             OGRARCGENLayer                              */
+/*                          OGRARCGENLayer                              */
 /************************************************************************/
 
-class OGRARCGENLayer final: public OGRLayer
+class OGRARCGENLayer final: public OGRLayer, public OGRGetNextFeatureThroughRaw<OGRARCGENLayer>
 {
     OGRFeatureDefn*    poFeatureDefn;
 
@@ -53,7 +53,7 @@ class OGRARCGENLayer final: public OGRLayer
                         virtual ~OGRARCGENLayer();
 
     virtual void                ResetReading() override;
-    virtual OGRFeature *        GetNextFeature() override;
+    DEFINE_GET_NEXT_FEATURE_THROUGH_RAW(OGRARCGENLayer)
 
     virtual OGRFeatureDefn *    GetLayerDefn() override { return poFeatureDefn; }
 
@@ -61,7 +61,7 @@ class OGRARCGENLayer final: public OGRLayer
 };
 
 /************************************************************************/
-/*                           OGRARCGENDataSource                           */
+/*                        OGRARCGENDataSource                           */
 /************************************************************************/
 
 class OGRARCGENDataSource final: public OGRDataSource
