@@ -1,5 +1,5 @@
 /******************************************************************************
-* $Id: ogr_fgdb.h c9bb0a71c27de43dab42821569ee6d9baf2fdf8f 2021-04-01 19:20:15 +0200 Even Rouault $
+* $Id: ogr_fgdb.h 5b3261184527657a6ffb2a6ec76055ae84001a09 2022-02-16 23:45:11 +0100 Even Rouault $
 *
 * Project:  OpenGIS Simple Features Reference Implementation
 * Purpose:  Standard includes and class definitions ArcObjects OGR driver.
@@ -108,6 +108,8 @@ class FGdbLayer final: public FGdbBaseLayer
 {
   friend class FGdbDataSource;
 
+  bool                m_bWorkaroundCrashOnCDFWithBinaryField = false;
+
   int                 m_bBulkLoadAllowed;
   int                 m_bBulkLoadInProgress;
 
@@ -276,18 +278,20 @@ protected:
 /************************************************************************/
 
 class FGdbDatabaseConnection;
+class OGRFileGDBGroup;
 
 class FGdbDataSource final: public OGRDataSource
 {
   CPLString             m_osFSName;
   CPLString             m_osPublicName;
   std::set<OGRLayer*>   m_oSetSelectLayers;
+  std::shared_ptr<GDALGroup>     m_poRootGroup{};
 
   int        FixIndexes();
   int        bPerLayerCopyingForTransaction;
 
 public:
-  FGdbDataSource(FGdbDriver* poDriver, FGdbDatabaseConnection* pConnection);
+  FGdbDataSource(bool bUseDriverMutex, FGdbDatabaseConnection* pConnection);
   virtual ~FGdbDataSource();
 
   int         Open(const char* pszFSName, int bUpdate,
@@ -313,6 +317,8 @@ public:
 
   const OGRFieldDomain* GetFieldDomain(const std::string& name) const override;
 
+  std::shared_ptr<GDALGroup> GetRootGroup() const override { return m_poRootGroup; }
+
   Geodatabase* GetGDB() { return m_pGeodatabase; }
   bool         GetUpdate() { return m_bUpdate; }
   FGdbDatabaseConnection* GetConnection() { return m_pConnection; }
@@ -335,15 +341,17 @@ public:
   */
 protected:
   bool LoadLayers(const std::wstring & parent);
-  bool OpenFGDBTables(const std::wstring &type,
+  bool OpenFGDBTables(OGRFileGDBGroup* group,
+                      const std::wstring &type,
                       const std::vector<std::wstring> &layers);
 
-  FGdbDriver* m_poDriver;
+  bool m_bUseDriverMutex = true;
   FGdbDatabaseConnection* m_pConnection;
-  std::vector <FGdbLayer*> m_layers;
+  std::vector <OGRLayer*> m_layers;
   Geodatabase* m_pGeodatabase;
   bool m_bUpdate;
   GDALDriver* m_poOpenFileGDBDrv;
+  std::unique_ptr<GDALDataset> m_poOpenFileGDBDS;
 };
 
 /************************************************************************/
@@ -374,28 +382,26 @@ public:
     void         CloseGeodatabase();
 };
 
-class FGdbDriver final: public OGRSFDriver, public IOGRTransactionBehaviour
+
+class FGdbTransactionManager final: public IOGRTransactionBehaviour
 {
-  std::map<CPLString, FGdbDatabaseConnection*> oMapConnections;
-  CPLMutex* hMutex;
-
 public:
-  FGdbDriver();
-  virtual ~FGdbDriver();
 
-  virtual const char *GetName() override;
-  virtual OGRDataSource *Open( const char *, int ) override;
-  virtual int TestCapability( const char * ) override;
-  virtual OGRDataSource *CreateDataSource( const char *pszName, char ** = nullptr) override;
-  virtual OGRErr DeleteDataSource( const char *pszDataSource ) override;
-
-  /* From IOGRTransactionBehaviour */
   virtual OGRErr StartTransaction(OGRDataSource*& poDSInOut, int& bOutHasReopenedDS) override;
   virtual OGRErr CommitTransaction(OGRDataSource*& poDSInOut, int& bOutHasReopenedDS) override;
   virtual OGRErr RollbackTransaction(OGRDataSource*& poDSInOut, int& bOutHasReopenedDS) override;
+};
 
-  void Release(const char* pszName);
-  CPLMutex* GetMutex() { return hMutex; }
+class FGdbDriver final: public GDALDriver
+{
+public:
+
+  static void Release(const char* pszName);
+
+  static FGdbTransactionManager* GetTransactionManager();
+
+  static CPLMutex* hMutex;
+  static FGdbTransactionManager* m_poTransactionManager;
 };
 
 CPL_C_START
