@@ -1,5 +1,5 @@
 /******************************************************************************
- * $Id: cpl_port.h 7a291205ed24a41c8f9a080fdd9630a6b565427d 2020-06-11 16:05:25 +0200 SpaceIm $
+ * $Id: cpl_port.h 9b04a6ce329ade6272643ab59d0ffaabd313955c 2021-10-11 23:32:03 +0200 Even Rouault $
  *
  * Project:  CPL - Common Portability Library
  * Author:   Frank Warmerdam, warmerdam@pobox.com
@@ -237,10 +237,6 @@ typedef int             GBool;
 /*      64bit support                                                   */
 /* -------------------------------------------------------------------- */
 
-#if defined(WIN32) && defined(_MSC_VER)
-#define VSI_LARGE_API_SUPPORTED
-#endif
-
 #if HAVE_LONG_LONG
 
 /** Large signed integer type (generally 64-bit integer type).
@@ -317,10 +313,6 @@ typedef unsigned int  GUIntptr_t;
 /** Printf formatting for GUIntBig */
 #define CPL_FRMT_GUIB    "%" CPL_FRMT_GB_WITHOUT_PREFIX "u"
 
-/*! @cond Doxygen_Suppress */
-#define GUINTBIG_TO_DOUBLE(x) CPL_STATIC_CAST(double, x)
-/*! @endcond */
-
 /*! @cond Doxygen_Suppress */
 #ifdef COMPAT_WITH_ICC_CONVERSION_CHECK
 #define CPL_INT64_FITS_ON_INT32(x) ((x) >= INT_MIN && (x) <= INT_MAX)
@@ -562,8 +554,8 @@ static inline char* CPL_afl_friendly_strstr(const char* haystack, const char* ne
 #endif /* defined(AFL_FRIENDLY) && defined(__GNUC__) */
 
 #  if defined(WIN32)
-#    define STRCASECMP(a,b)         (stricmp(a,b))
-#    define STRNCASECMP(a,b,n)      (strnicmp(a,b,n))
+#    define STRCASECMP(a,b)         (_stricmp(a,b))
+#    define STRNCASECMP(a,b,n)      (_strnicmp(a,b,n))
 #  else
 /** Alias for strcasecmp() */
 #    define STRCASECMP(a,b)         (strcasecmp(a,b))
@@ -641,7 +633,7 @@ static inline int CPLIsFinite(double f) { return !__isnan(f) && !__isinf(f); }
 #else
 #  define CPLIsNan(x) isnan(x)
 #  if defined(isinf) || defined(__FreeBSD__)
-/** Return whether a floating-pointer number is +/- infinty */
+/** Return whether a floating-pointer number is +/- infinity */
 #    define CPLIsInf(x) isinf(x)
 /** Return whether a floating-pointer number is finite */
 #    define CPLIsFinite(x) (!isnan(x) && !isinf(x))
@@ -709,11 +701,7 @@ template<> struct CPLStaticAssert<true>
 /** Byte-swap a 16bit unsigned integer */
 #define CPL_SWAP16(x) CPL_STATIC_CAST(GUInt16, (CPL_STATIC_CAST(GUInt16, x) << 8) | (CPL_STATIC_CAST(GUInt16, x) >> 8) )
 
-#if defined(HAVE_GCC_BSWAP) && (defined(__i386__) || defined(__x86_64__))
-/* Could potentially be extended to other architectures but must be checked */
-/* that the intrinsic is indeed efficient */
-/* GCC (at least 4.6  or above) need that include */
-#include <x86intrin.h>
+#if defined(HAVE_GCC_BSWAP)
 /** Byte-swap a 32bit unsigned integer */
 #define CPL_SWAP32(x) CPL_STATIC_CAST(GUInt32, __builtin_bswap32(CPL_STATIC_CAST(GUInt32, x)))
 /** Byte-swap a 64bit unsigned integer */
@@ -739,76 +727,36 @@ template<> struct CPLStaticAssert<true>
 
 /** Byte-swap a 16 bit pointer */
 #define CPL_SWAP16PTR(x) \
-{                                                                 \
-    GByte       byTemp, *_pabyDataT = CPL_REINTERPRET_CAST(GByte*, x);              \
+do {                                                                        \
+    GUInt16 _n16;                                                           \
+    void* _lx = x;                                                          \
+    memcpy(&_n16, _lx, 2);                                                  \
     CPL_STATIC_ASSERT_IF_AVAILABLE(sizeof(*(x)) == 1 || sizeof(*(x)) == 2); \
-                                                                  \
-    byTemp = _pabyDataT[0];                                       \
-    _pabyDataT[0] = _pabyDataT[1];                                \
-    _pabyDataT[1] = byTemp;                                       \
-}
-
-#if defined(MAKE_SANITIZE_HAPPY) || !(defined(__i386__) || defined(__x86_64__) || defined(_M_IX86) || defined(_M_X64))
+    _n16 = CPL_SWAP16(_n16);                                                \
+    memcpy(_lx, &_n16, 2);                                                  \
+} while(0)
 
 /** Byte-swap a 32 bit pointer */
 #define CPL_SWAP32PTR(x) \
-{                                                                 \
-    GByte       byTemp, *_pabyDataT = CPL_REINTERPRET_CAST(GByte*, x);              \
-    CPL_STATIC_ASSERT_IF_AVAILABLE(sizeof(*(x)) == 1 || sizeof(*(x)) == 4);  \
-                                                                  \
-    byTemp = _pabyDataT[0];                                       \
-    _pabyDataT[0] = _pabyDataT[3];                                \
-    _pabyDataT[3] = byTemp;                                       \
-    byTemp = _pabyDataT[1];                                       \
-    _pabyDataT[1] = _pabyDataT[2];                                \
-    _pabyDataT[2] = byTemp;                                       \
-}
-
-/** Byte-swap a 64 bit pointer */
-#define CPL_SWAP64PTR(x) \
-{                                                                 \
-    GByte       byTemp, *_pabyDataT = CPL_REINTERPRET_CAST(GByte*, x);              \
-    CPL_STATIC_ASSERT_IF_AVAILABLE(sizeof(*(x)) == 1 || sizeof(*(x)) == 8); \
-                                                                  \
-    byTemp = _pabyDataT[0];                                       \
-    _pabyDataT[0] = _pabyDataT[7];                                \
-    _pabyDataT[7] = byTemp;                                       \
-    byTemp = _pabyDataT[1];                                       \
-    _pabyDataT[1] = _pabyDataT[6];                                \
-    _pabyDataT[6] = byTemp;                                       \
-    byTemp = _pabyDataT[2];                                       \
-    _pabyDataT[2] = _pabyDataT[5];                                \
-    _pabyDataT[5] = byTemp;                                       \
-    byTemp = _pabyDataT[3];                                       \
-    _pabyDataT[3] = _pabyDataT[4];                                \
-    _pabyDataT[4] = byTemp;                                       \
-}
-
-#else
-
-/** Byte-swap a 32 bit pointer */
-#define CPL_SWAP32PTR(x) \
-{                                                                           \
+do {                                                                        \
     GUInt32 _n32;                                                           \
     void* _lx = x;                                                          \
     memcpy(&_n32, _lx, 4);                                                  \
     CPL_STATIC_ASSERT_IF_AVAILABLE(sizeof(*(x)) == 1 || sizeof(*(x)) == 4); \
     _n32 = CPL_SWAP32(_n32);                                                \
     memcpy(_lx, &_n32, 4);                                                  \
-}
+} while(0)
 
 /** Byte-swap a 64 bit pointer */
 #define CPL_SWAP64PTR(x) \
-{                                                                           \
+do {                                                                        \
     GUInt64 _n64;                                                           \
     void* _lx = x;                                                          \
     memcpy(&_n64, _lx, 8);                                                    \
     CPL_STATIC_ASSERT_IF_AVAILABLE(sizeof(*(x)) == 1 || sizeof(*(x)) == 8); \
     _n64 = CPL_SWAP64(_n64);                                                \
     memcpy(_lx, &_n64, 8);                                                    \
-}
-
-#endif
+} while(0)
 
 /** Byte-swap a 64 bit pointer */
 #define CPL_SWAPDOUBLE(p) CPL_SWAP64PTR(p)
@@ -1010,7 +958,7 @@ static const char *cvsid_aw() { return( cvsid_aw() ? NULL : cpl_cvsid ); }
 
 #endif /* __cplusplus */
 
-#if !defined(DOXYGEN_SKIP)
+#if !defined(DOXYGEN_SKIP) && !defined(CPL_WARN_DEPRECATED)
 #if defined(__has_extension)
   #if __has_extension(attribute_deprecated_with_message)
     /* Clang extension */
@@ -1051,10 +999,15 @@ CPL_C_END
 #endif
 
 #if defined(__cplusplus)
+#ifndef CPPCHECK
 /** Returns the size of C style arrays. */
 #define CPL_ARRAYSIZE(array) \
   ((sizeof(array) / sizeof(*(array))) / \
   static_cast<size_t>(!(sizeof(array) % sizeof(*(array)))))
+#else
+/* simplified version for cppcheck */
+#define CPL_ARRAYSIZE(array) (sizeof(array) / sizeof(array[0]))
+#endif
 
 extern "C++" {
 template<class T> static void CPL_IGNORE_RET_VAL(const T&) {}
@@ -1204,7 +1157,7 @@ inline C CPLUnsanitizedAdd(A a, B b)
 /* This typedef is for C functions that take char** as argument, but */
 /* with the semantics of a const list. In C, char** is not implicitly cast to */
 /* const char* const*, contrary to C++. So when seen for C++, it is OK */
-/* to expose the prototyes as const char* const*, but for C we keep the */
+/* to expose the prototypes as const char* const*, but for C we keep the */
 /* historical definition to avoid warnings. */
 #if defined(__cplusplus) && !defined(CPL_SUPRESS_CPLUSPLUS) && !defined(DOXYGEN_SKIP)
 /** Type of a constant null-terminated list of nul terminated strings.
