@@ -4,11 +4,11 @@
  * Purpose:  Implementation of quadtree building and searching functions.
  *           Derived from shapelib and mapserver implementations
  * Author:   Frank Warmerdam, warmerdam@pobox.com
- *           Even Rouault, <even dot rouault at mines dash paris dot org>
+ *           Even Rouault, <even dot rouault at spatialys.com>
  *
  ******************************************************************************
  * Copyright (c) 1999-2008, Frank Warmerdam
- * Copyright (c) 2008-2014, Even Rouault <even dot rouault at mines-paris dot org>
+ * Copyright (c) 2008-2014, Even Rouault <even dot rouault at spatialys.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -39,7 +39,7 @@
 #include "cpl_conv.h"
 #include "cpl_error.h"
 
-CPL_CVSID("$Id: cpl_quad_tree.cpp 5e0bc19589a7902ad40bd8c2ed9af34f8e62a9b4 2018-07-04 22:34:59 +0200 Even Rouault $")
+CPL_CVSID("$Id: cpl_quad_tree.cpp 79536ea3fcf18af9a8437aa30640e9d46d027787 2021-07-25 20:04:54 +0200 Even Rouault $")
 
 constexpr int MAX_DEFAULT_TREE_DEPTH = 12;
 constexpr int MAX_SUBNODES = 4;
@@ -74,6 +74,7 @@ struct _CPLQuadTree
 static void CPLQuadTreeAddFeatureInternal(CPLQuadTree *hQuadTree,
                                           void* hFeature,
                                           const CPLRectObj *pRect);
+static void CPLQuadTreeNodeDestroy(QuadTreeNode *psNode);
 
 /* -------------------------------------------------------------------- */
 /*      If the following is 0.5, psNodes will be split in half.  If it  */
@@ -301,6 +302,105 @@ void CPLQuadTreeInsertWithBounds(CPLQuadTree *hQuadTree,
     CPLQuadTreeAddFeatureInternal(hQuadTree, hFeature, psBounds);
 }
 
+/************************************************************************/
+/*                            CPLQuadTreeRemove()                       */
+/************************************************************************/
+
+static bool CPLQuadTreeRemoveInternal(QuadTreeNode* psNode,
+                                      void* hFeature,
+                                      const CPLRectObj* psBounds)
+{
+    bool bRemoved = false;
+
+    for( int i = 0; i < psNode->nFeatures; i++ )
+    {
+        if( psNode->pahFeatures[i] == hFeature )
+        {
+            if( i < psNode->nFeatures - 1 )
+            {
+                memmove(psNode->pahFeatures + i,
+                        psNode->pahFeatures + i + 1,
+                        (psNode->nFeatures - 1 - i) * sizeof(void*));
+                if( psNode->pasBounds )
+                {
+                    memmove(psNode->pasBounds + i,
+                            psNode->pasBounds + i + 1,
+                            (psNode->nFeatures - 1 - i) * sizeof(CPLRectObj));
+                }
+            }
+            bRemoved = true;
+            psNode->nFeatures --;
+            break;
+        }
+    }
+    if( psNode->nFeatures == 0 && psNode->pahFeatures != nullptr )
+    {
+        CPLFree(psNode->pahFeatures);
+        CPLFree(psNode->pasBounds);
+        psNode->pahFeatures = nullptr;
+        psNode->pasBounds = nullptr;
+    }
+
+    /* -------------------------------------------------------------------- */
+    /*      Recurse to subnodes if they exist.                              */
+    /* -------------------------------------------------------------------- */
+    for( int i = 0; i < psNode->nNumSubNodes; i++ )
+    {
+        if( psNode->apSubNode[i] &&
+            CPL_RectOverlap(&(psNode->apSubNode[i]->rect), psBounds) )
+        {
+            bRemoved |=
+                CPLQuadTreeRemoveInternal( psNode->apSubNode[i], hFeature, psBounds );
+
+            if( psNode->apSubNode[i]->nFeatures == 0 &&
+                psNode->apSubNode[i]->nNumSubNodes == 0 )
+            {
+                CPLQuadTreeNodeDestroy(psNode->apSubNode[i]);
+                if( i < psNode->nNumSubNodes - 1 )
+                {
+                    memmove(psNode->apSubNode + i,
+                            psNode->apSubNode + i + 1,
+                            (psNode->nNumSubNodes - 1 - i) * sizeof(QuadTreeNode*));
+                }
+                i --;
+                psNode->nNumSubNodes --;
+            }
+        }
+    }
+
+    return bRemoved;
+}
+
+/**
+ * Remove a feature from a quadtree.
+ *
+ * Currently the quadtree is not re-balanced.
+ *
+ * @param hQuadTree the quad tree
+ * @param hFeature the feature to remove
+ * @param psBounds bounds of the feature (or NULL if pfnGetBounds has been filled)
+ */
+void CPLQuadTreeRemove(CPLQuadTree *hQuadTree, void* hFeature,
+                       const CPLRectObj* psBounds)
+{
+    if( psBounds == nullptr && hQuadTree->pfnGetBounds == nullptr )
+    {
+        CPLError(CE_Failure, CPLE_AppDefined,
+                 "hQuadTree->pfnGetBounds == NULL");
+        return;
+    }
+    CPLRectObj bounds; // keep variable in this outer scope
+    if( psBounds == nullptr )
+    {
+        hQuadTree->pfnGetBounds(hFeature, &bounds);
+        psBounds = &bounds;
+    }
+    if( CPLQuadTreeRemoveInternal(hQuadTree->psRoot, hFeature, psBounds) )
+    {
+        hQuadTree->nFeatures--;
+    }
+}
+
 /************************************************************************/
 /*                    CPLQuadTreeNodeDestroy()                          */
 /************************************************************************/
