@@ -27,7 +27,7 @@
 
 #include <algorithm>
 
-CPL_CVSID("$Id: cpl_recode_iconv.cpp 07d22b7d8a4f947d2fb9bbab1ef546c6ade46d00 2021-11-20 17:01:02 +0100 Even Rouault $")
+CPL_CVSID("$Id: cpl_recode_iconv.cpp a291c9699f33b9a9eca01b7e008dff4e46a5cf2a 2022-03-30 15:25:46 +0200 Stefan Uhrig $")
 
 #ifdef CPL_RECODE_ICONV
 
@@ -256,7 +256,7 @@ char *CPLRecodeFromWCharIconv( const wchar_t *pwszSource,
         reinterpret_cast<char*>(pszIconvSrcBuf));
 
     /* iconv expects a number of bytes, not characters */
-    nSrcLen *= sizeof(wchar_t);
+    nSrcLen *= nTargetCharWidth;
 
 /* -------------------------------------------------------------------- */
 /*      Allocate destination buffer.                                    */
@@ -277,8 +277,8 @@ char *CPLRecodeFromWCharIconv( const wchar_t *pwszSource,
             if( errno == EILSEQ )
             {
                 // Skip the invalid sequence in the input string.
-                nSrcLen--;
-                pszSrcBuf += sizeof(wchar_t);
+                nSrcLen -= nTargetCharWidth;
+                pszSrcBuf += nTargetCharWidth;
                 if( !bHaveWarned2 )
                 {
                     bHaveWarned2 = true;
@@ -309,6 +309,13 @@ char *CPLRecodeFromWCharIconv( const wchar_t *pwszSource,
         }
     }
 
+    if (nDstLen == 0)
+    {
+        ++nDstCurLen;
+        pszDestination =
+            static_cast<char *>(CPLRealloc(pszDestination, nDstCurLen));
+        ++nDstLen;
+    }
     pszDestination[nDstCurLen - nDstLen] = '\0';
 
     iconv_close( sConv );
